// Persistence Of Vision raytracer version 3.5 sample file.
// POV-Ray scene description for chess board.
// By Ville Saari
// Copyright (c) 1991 Ferry Island Pixelboys
//
// This scene has 430 primitives in objects and 41 in bounding shapes and
// it takes over 40 hours to render by standard amiga.
//
// If you do some nice modifications or additions to this file, please send
// me a copy. My Internet address is:  vsaari@niksula.hut.fi

// Note : CHESS2.POV was created from Ville Saari's chess.pov
// -- Dan Farmer 1996
//  - Cchanged textures
//  - Added camera blur and changed focal length
//  - Use sky sphere
//  - Modularized the code
//  - Added felt pads to bottom of pieces

// remaining manual bounding commented out by Bob Hughes, August 31, 2001

// modified into a checkers board by Yves Debilloz


global_settings { assumed_gamma 2.2 max_trace_level 5 }

#include "shapes.inc"
#include "colors.inc"
#include "textures.inc"
#include "skies.inc"
#include "metals.inc"
#include "woods.inc"

#declare FB_Quality_Off     =  0;
#declare FB_Quality_Fast    =  1;
#declare FB_Quality_Default =  2;
#declare FB_Quality_High    =  3;

#declare FB_Quality= FB_Quality_Off;

camera {
   location <0, 50, -90>
   direction <0, 0, 2>
   up <0, 1, 0>
   right <4/3, 0, 0>
   look_at <0, -1.25, 1.5>

#if(FB_Quality != FB_Quality_Off)
      focal_point <0, 0, 0>
#end

#switch(FB_Quality)
#case(FB_Quality_Off)
   aperture 0
        #debug "\nNo focal blur used...\n"
#break
#case (FB_Quality_Fast)
   aperture 2.25
   blur_samples 7
   confidence 0.5             // default is 0.9
   variance 1/64              // default is 1/128 (0.0078125)
        #debug "\nFast focal blur used...\n"
#break
#case(FB_Quality_Default)
   aperture 2.25
   blur_samples 19
   confidence 0.90            // default is 0.9
   variance 1/128             // default is 1/128 (0.0078125)
        #debug "\nDefault focal blur used...\n"
#break
#case(FB_Quality_High)
   aperture 1
   blur_samples 37
   confidence 0.975           // default is 0.9
   variance 1/1024            // default is 1/128 (0.0078125)
        #debug "\nHigh Quality focal blur used...\n"
#break
#else
        #debug "\nNo focal blur used...\n"
#end
}

light_source { <-2000, 1600, 200> colour White }
light_source { <2000, 1600, 200> colour White }
light_source { <2000, 1600, -200> colour White }
light_source { <-2000, 1600, -200> colour White }


#declare PawnBase =
union {
/*
    intersection {
       sphere { <0, 0, 0>, 0.5 }
       plane { -y, 0 }
    }
*/
    cylinder { 0, y*1.1, 2.5 }
}

#declare WWood = texture {
    White_Wood
    //T_Silver_3B
}

#declare BWood = texture {
    Dark_Wood
    //T_Gold_3C
}

#declare WPawn = object {
   PawnBase

   texture {
      WWood
      pigment { quick_color red 0.95 green 0.62 }
   }
}

#declare BPawn = object {
   PawnBase

   texture {
      BWood
      pigment { quick_color red 0.4 green 0.2 }
   }
}

/* Sky */
#declare SkySphere = sky_sphere { S_Cloud1 }

/* Ground */
#declare Ground =
plane { y, -80
   pigment { green 0.20 }
   finish {
      ambient 0.25
      diffuse 0.5
   }
}

#declare FarSide =
union {
   object { BPawn translate <-28, 0, 20> }
   object { BPawn translate <-20, 0, 28> }
   object { BPawn translate <-12, 0, 20> }
   object { BPawn translate < -4, 0, 28> }
   object { BPawn translate <  4, 0, 20> }
   object { BPawn translate < 12, 0, 28> }
   object { BPawn translate < 20, 0, 20> }
   object { BPawn translate < 28, 0, 28> }

   object { BPawn translate <-20, 0, 12> }
   object { BPawn translate < -4, 0, 12> }
   object { BPawn translate < 12, 0, 12> }
   object { BPawn translate < 28, 0, 12> }

}

#declare NearSide =
union {
   object { WPawn translate <-28, 0, -28> }
   object { WPawn translate <-20, 0, -20> }
   object { WPawn translate <-12, 0, -28> }
   object { WPawn translate < -4, 0, -20> }
   object { WPawn translate <  4, 0, -28> }
   object { WPawn translate < 12, 0, -20> }
   object { WPawn translate < 20, 0, -28> }
   object { WPawn translate < 28, 0, -20> }

   object { WPawn translate <-28, 0, -12> }
   object { WPawn translate <-12, 0, -12> }
   object { WPawn translate <  4, 0, -12> }
   object { WPawn translate < 20, 0, -12> }

}

#declare Pieces =
union {
   object { NearSide }
   object { FarSide }
}

#declare FramePiece =
intersection {
   plane { +y, -0.15 }
   plane { -y, 3 }
   plane { -z, 35 }
   plane { <-1, 0, 1>, 0 }      // 45 degree bevel
   plane { < 1, 0, 1>, 0 }      // 45 degree bevel
}

#declare Frame =
union {
   union {
      object { FramePiece }
      object { FramePiece rotate 180*y }
      texture {
         T_Wood20
         scale 2
         rotate y*87
         translate x*1
         finish {
            specular 1
            roughness 0.02
            ambient 0.35
          }
      }
   }

   union {
      object { FramePiece rotate -90*y }
      object { FramePiece rotate  90*y }
      texture {
         T_Wood20
         scale 2
         rotate y*2
         finish {
            specular 1
            roughness 0.02
            ambient 0.35
          }
      }
   }
}

#declare Board =
   box { <-32, -1, -32> <32, 0, 32>
      texture {
         tiles {
            texture {
               pigment {
                  //marble
                  wrinkles
                  turbulence 1.0
                  colour_map {
                     [0.0 0.7 colour White
                              colour White]
                     [0.7 0.9 colour White
                              colour red 0.8 green 0.8 blue 0.8]
                     [0.9 1.0 colour red 0.8 green 0.8 blue 0.8
                              colour red 0.5 green 0.5 blue 0.5]
                  }
                  scale <0.6, 1, 0.6>
                  rotate -30*y
               }
               finish {
                  specular 1
                  roughness 0.02
                  reflection 0.125
               }
            } // texture
            tile2
            texture {
               pigment {
                  granite
                  scale <0.3, 1, 0.3>
                  colour_map {
                     [0 1 colour Black
                          colour red 0.5 green 0.5 blue 0.5]
                  }
               }
               finish {
                  specular 1
                  roughness 0.02
                  reflection 0.125
               }
            }
         } // texture
         scale <8, 1, 8>
      } //texture
   } // intersection



/* Table */
#declare Table =
union {
   intersection {
      plane { +y, -3 }
      plane { -y,  8 }
      sphere { <0, -5.5, 0>, 55 }
   }

   intersection {
      plane { y, -8 }
      object {
         Hyperboloid_Y
         scale <10, 20, 10>
         translate -20*y
      }
   }

   pigment {
      granite
      scale 6
   }
   finish {
      specular 1
      roughness 0.02
      reflection 0.5
   }
}

object { Pieces }
object { Board }
object { Frame }
object { Ground }
object { Table }
sky_sphere { SkySphere }
