/*

Goban kit for POV-Ray 3.5, 2002
by Tuomas "pto" Ojamies, 12 kyu
ojamies@cs.helsinki.fi
http://www.cs.helsinki.fi/u/ojamies/

Howto use?
	Include this file, declare parameters that you want to change,
	create your own wood texture, call othello() macro and place the 
	stones in one way or other.

Parameters?
	border
		empty space around the grid
	line_width
		width of lines
	hoshi_width
		radius or diameter of hoshi points
	grid_x
		size of the grid along the x axis
	grid_y
		size of the grid along the z axis
	grid_s
		number of lines parallel in grid
	board_height
		height of the board

	stone_rad
		radius of stone
	stone_htr_ratio
		determines height of stone from its radiu
	stone_top_roundnes
		controls the roundness of top
	stone_edge_roundnes
		constrols the roundless of edges 


Textures?
	t_board
		wood texture to the board, grid is automagically placed after this
		so just make nice wood
	t_blackstone and t_whitestone
		textures for black and white stones

Placing the stones?
	Placing 200+ stones by hand may be cool but I recommend using some 
	program to generate macro calls that place the stones or exporting 
	game to ascii diagram and reading it.

	board_coords(x,z)
		determines coordinates of given point in the grid, y component is
		the height of the board. Parameter should take values from 0 to
		grid_s-1 to hit the crospoints.
	b_stone and w_stone
		black and white stone, just translate these with vector given by
		the board_coords macro.

Something else?
	If grid_s is even, hoshi points will be only in corners. Same aplies
	if grid_s is smaler than nine. Note, it is not decent to have goban
	without tengen (centerpoint)! Hoshi points jump to third line when
	grid_s is smaler than 12.

	Remember to use heavy antialiasing if line_width is thin.

*/

#declare border = <1.38,0,1.38>;
#declare line_width = .10;
#declare hoshi_width = .1;
#declare grid_x = 44.07;
#declare grid_z = 41.04;
#declare grid_s = 8;
#declare board_height = 1;

#declare stone_rad = 2.5;
#declare stone_hgt = .75;
#declare stone_top_roundnes = .3;
#declare stone_edge_roundnes = .4;

#declare grid_finish =
finish {}

#declare mf_wood =
function {
	.5+.5*sin(35*x)
}
#declare mf_granite =
function {
	pigment { granite }
}

#declare t_blackstone =
texture {
	pigment{rgb .1}
	finish {specular .2}
}

#declare t_whitestone =
texture {
	pigment{rgb 1}
	finish {diffuse .8 specular .2}
}

#declare t_board =
texture {
	pigment { 
		function { 
			.1*mf_wood(x,y,z)+.4*mf_granite(.25*x,.05*y,.05*z).y+.3*mf_granite(5*x,y,z).y
		} 
		color_map {
                        [0.6  color Green]
			[0 rgb <.6,.4,.2>]
			[1 rgb <.9,.7,.3>]
		}
		turbulence .1 
	}
	normal {granite .1 scale <1,1,3>}
	finish {
		specular .2
	}
}

#macro othello()

#declare f_line =
function {select (abs(x)-line_width/2,.3,0)}

#declare f_hoshi = 
function (x,y,z,hx,hz) {
	select(
		pow((x-(grid_x-line_width)/2+hx/(grid_s-1)*(grid_x-line_width)),2)+
		pow((z-(grid_z-line_width)/2+hz/(grid_s-1)*(grid_z-line_width)),2)-

		hoshi_width/2,
		.3,0
	)
}

#declare grid_m = div(grid_s,2);

#declare t_grid =
	texture {
		pigment {
			function {
				0
				#declare i=0;#declare n=grid_s-1;#while(i<n+1)

				+f_line(x-(grid_x-line_width)/2+i/n*(grid_x-line_width),y,z)
				+f_line(z-(grid_z-line_width)/2+i/n*(grid_z-line_width),y,x)

				#declare i=i+1;#end

				#if(grid_s>12)
				
				+f_hoshi(x,y,z,grid_s-4,3)
				+f_hoshi(x,y,z,grid_s-4,grid_s-4)
				+f_hoshi(x,y,z,3,3)
				+f_hoshi(x,y,z,3,grid_s-4)

				#if(mod(grid_s,2)=1)
				+f_hoshi(x,y,z,grid_m,grid_m)
				+f_hoshi(x,y,z,3,grid_m)
				+f_hoshi(x,y,z,grid_m,3)
				+f_hoshi(x,y,z,grid_m,grid_s-4)
				+f_hoshi(x,y,z,grid_s-4,grid_m)
				#end

				#else

                +f_hoshi(x,y,z,grid_s-3,2)
                +f_hoshi(x,y,z,grid_s-3,grid_s-3)
                +f_hoshi(x,y,z,2,2)
                +f_hoshi(x,y,z,2,grid_s-3)

				#if(grid_s = 9 | grid_s = 11)
                +f_hoshi(x,y,z,grid_m,grid_m)
                +f_hoshi(x,y,z,2,grid_m)
                +f_hoshi(x,y,z,grid_m,2)
                +f_hoshi(x,y,z,grid_m,grid_s-3)
                +f_hoshi(x,y,z,grid_s-3,grid_m)

				#end

				#end

			}
			color_map {[0 rgbt 1][.3 rgbt 0][1 rgbt 0]}
		}
	}

union {
	difference {
	    box {<-grid_x/2,0,-grid_z/2>-border<grid_x/2,board_height,grid_z/2>+border}
    	box {<-grid_x/2,-1,-grid_z/2><grid_x/2,board_height+1,grid_z/2>}
	    texture {t_board}
	}
	box {
	    <-grid_x/2,0,-grid_z/2><grid_x/2,board_height,grid_z/2>
	    texture {t_board}
	    texture {t_grid}
	}		
}


#declare stone =
    lathe {
        bezier_spline 8
        <0,stone_hgt/2>
		<stone_top_roundnes*stone_rad,stone_hgt/2>
		<stone_rad,stone_edge_roundnes*stone_hgt/2> 
		<stone_rad,0>

		<0,-stone_hgt/2>
		<stone_top_roundnes*stone_rad,-stone_hgt/2>
		<stone_rad,-stone_edge_roundnes*stone_hgt/2>
		<stone_rad,0>
		translate y*stone_hgt/2
    }

#declare b_stone =
	object { stone texture { t_blackstone } }

#declare w_stone =
	object { stone texture { t_whitestone } }

#end	

#macro board_coords(f,g)
	<-(grid_x-line_width)/2+(f-1)/(grid_s-1)*(grid_x-line_width),
	 board_height,
	 -(grid_z-line_width)/2+(g-1)/(grid_s-1)*(grid_z-line_width)>
#end

#macro stone_radius()
	min(grid_x,grid_z)/(grid_s-1)
#end

