//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "CheckersEng.h"
#include <stdio.h>

#pragma package(smart_init)

//---------------------------------------------------------------------------

static bool bValidCase[] =
                        { true,false,true,false,true,false,true,false ,
                          false,true,false,true,false,true,false,true ,
                          true,false,true,false,true,false,true,false ,
                          false,true,false,true,false,true,false,true ,
                          true,false,true,false,true,false,true,false ,
                          false,true,false,true,false,true,false,true ,
                          true,false,true,false,true,false,true,false ,
                          false,true,false,true,false,true,false,true
                        };

//---------------------------------------------------------------------------

__fastcall CheckersEngine::CheckersEngine(TAlgorithmType nType)
  : EngCheckersBase(nType)
{
  // make sure the priority is high enough to get run before
  // the UI task.  we will call Suspended from the main thread
  // regularly and if the priority is the same as the main thread
  // this would be highly inefficient

  Depth = 6;
  Fields = 64;
}

// ---------------------------------------------------------------------------
// empty the board to its initial status

void CheckersEngine::InitBoard(void)
{
  int i;

  EngCheckersBase::InitBoard();
  BoardDimension = 8;

  for (i=0;i<=23;i++)
    if (bValidCase[i]) SetPieceAt(i,1);
  for (i=40;i<=63;i++)
    if (bValidCase[i]) SetPieceAt(i,2);
}

// ---------------------------------------------------------------------------
// check if we could move a stone from a certain position
// and add all possible options to the list of possible moves
// if there is already one capture move, do not add futher
// regular moves to the list

bool CheckersEngine::AddValidFrom(const int nPos, const int nOwnColour,
                                  TMoveList *aMoves,const int nMode)
{
  bool bFoundCaptureMove;
  int  nStoneColour = aBoard[Board_Hor(nPos)][Board_Ver(nPos)];

  if ((nStoneColour & 3) != nOwnColour)
    return false; // no own stone here, so no capture move found

  if (nVariant == 0)
  {
    // american checkers form
    if (nMode == ALL_MOVE_TYPES)
    {
      if (nStoneColour==1)
      {
        // normal white stone
        // allow forward normal moves and backwards captures
        bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,1,1,true);
        bFoundCaptureMove = bFoundCaptureMove || AddValidInDir(nPos,nOwnColour,aMoves,-1,1,true);
        bFoundCaptureMove = bFoundCaptureMove || AddValidInDir(nPos,nOwnColour,aMoves,1,-1,false);
        bFoundCaptureMove = bFoundCaptureMove || AddValidInDir(nPos,nOwnColour,aMoves,-1,-1,false);
      }
      else if (nStoneColour==2)
      {
        // normal black stone
        // allow forward normal moves and backwards captures
        bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,1,-1,true);
        bFoundCaptureMove = bFoundCaptureMove || AddValidInDir(nPos,nOwnColour,aMoves,-1,-1,true);
        bFoundCaptureMove = bFoundCaptureMove || AddValidInDir(nPos,nOwnColour,aMoves,1,1,false);
        bFoundCaptureMove = bFoundCaptureMove || AddValidInDir(nPos,nOwnColour,aMoves,-1,1,false);
      }
      else
      {
        // queen stone
        // allow forward normal moves and backwards moves
        bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,1,1,true);
        bFoundCaptureMove = bFoundCaptureMove || AddValidInDir(nPos,nOwnColour,aMoves,-1,1,true);
        bFoundCaptureMove = bFoundCaptureMove || AddValidInDir(nPos,nOwnColour,aMoves,1,-1,true);
        bFoundCaptureMove = bFoundCaptureMove || AddValidInDir(nPos,nOwnColour,aMoves,-1,-1,true);
      }
    }
    else
    {
      // allow captures only
      bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,1,1,false);
      bFoundCaptureMove = bFoundCaptureMove || AddValidInDir(nPos,nOwnColour,aMoves,-1,1,false);
      bFoundCaptureMove = bFoundCaptureMove || AddValidInDir(nPos,nOwnColour,aMoves,1,-1,false);
      bFoundCaptureMove = bFoundCaptureMove || AddValidInDir(nPos,nOwnColour,aMoves,-1,-1,false);
    }
  }
  else
  {
    // simple checkers form
    bool lType = (nMode == ALL_MOVE_TYPES);

    if (nStoneColour==1)
    {
      // normal white stone
      // allow forward normal moves and backwards captures
      bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,1,1,lType);
      bFoundCaptureMove = bFoundCaptureMove || AddValidInDir(nPos,nOwnColour,aMoves,-1,1,lType);
    }
    else if (nStoneColour==2)
    {
      // normal black stone
      // allow forward normal moves and backwards captures
      bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,1,-1,lType);
      bFoundCaptureMove = bFoundCaptureMove || AddValidInDir(nPos,nOwnColour,aMoves,-1,-1,lType);
    }
    else
    {
      // queen stone
      // allow forward normal moves and backwards moves
      // or condition at end, otherwise the optimizer removes the
      // second capture move with the same piece
      bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,1,1,lType);
      bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,-1,1,lType) ||
                          bFoundCaptureMove ;
      bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,1,-1,lType) ||
                          bFoundCaptureMove ;
      bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,-1,-1,lType) ||
                          bFoundCaptureMove ;
    }
  }

  return bFoundCaptureMove;  // return true for capture move, false for other
}

// ---------------------------------------------------------------------------
// add move from pos in direction to movelist
// will possibly convert from allmovetypes to captures only
// by returning the captures only flag

bool inline CheckersEngine::AddValidInDir(const int nPos,
      const int nOwnColour, TMoveList * aMoves,
      const int nDirHor, const int nDirVer, const bool bAllMoveTypes)
{
  int bFoundCaptureMove = false;
  int nHor = Board_Hor(nPos);
  int nVer = Board_Ver(nPos);
  int nNewVer = nVer+nDirVer;
  int nNewHor = nHor+nDirHor;

  // only one test is needed but we do not know if we are going forward
  // or backward, so just test both

  if ((nNewVer >= 0) && (nNewVer<BoardDimension)
       && (nNewHor >= 0) && (nNewHor<BoardDimension))
  {
    if (aBoard[nNewHor][nNewVer]==0)
    {
      // empty case, so normal move is possible
      if (bAllMoveTypes)
        // we are seeking for all moves, not only
        // just the capture moves
        AddValidMoveToList(aMoves,nPos,nNewHor+nNewVer*BoardDimension,false);
    }
    else if ((aBoard[nNewHor][nNewVer]&3)!=nOwnColour)
    {
      nNewVer += nDirVer;
      nNewHor += nDirHor;
      if ((nNewVer >= 0) && (nNewVer<BoardDimension)
          && (nNewHor >= 0) && (nNewHor<BoardDimension))
      {
        // we are still on the board, so the move is valid
        if (aBoard[nNewHor][nNewVer]==0)
        {
          // and square we want to jump too is empty
          // so store this capture move
          AddValidMoveToList(aMoves,nPos,(nNewHor)+(nNewVer)*BoardDimension,true);
          bFoundCaptureMove = true;
        }
      }
    }
  }
  return bFoundCaptureMove;
}

// ---------------------------------------------------------------------------
// calculate the value of the position

signed long int CheckersEngine::EvaluatePos(const int nColour)
{
  register int i,j;

  signed long int liValue = 0;
  signed long int aValues[] = { 0 , 10 , -10 , 0 , 0 , 20 , -20 };
  // allow space for normal pieces and for queens

  for (i=0;i<BoardDimension;i++)
    for (j=0;j<BoardDimension;j++)
      if (aBoard[i][j])
        liValue += (aValues[aBoard[i][j]]);

  return nColour==1?liValue:-liValue;
}

// ---------------------------------------------------------------------------
// eof






