//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "EngConnect4.h"
#include <stdio.h>
#pragma package(smart_init)

//---------------------------------------------------------------------------

__fastcall Connect4Eng::Connect4Eng(TAlgorithmType nType)
  : GameEngine(nType)
{
  // make sure the priority is high enough to get run before
  // the UI task.  we will call Suspended from the main thread
  // regularly and if the priority is the same as the main thread
  // this would be highly inefficient

  Depth = 6;
  Fields = 42;
  MaxUniqueMoves = 7;
  BoardDimension = 7;
}

// ---------------------------------------------------------------------------
// generate a list of valid moves

int Connect4Eng::GenerateMoves(TMoveList *aMoves,const int nColour)
{
  register int i,j;
  int nFound = 0;
  int aColumns[7] = { 3,4,2,5,1,6,0 };

  // see if can take and add to list of valid moves
  for (j=0;j<7;j++)
  {
    for (i=0;i<Fields;i++)
      if (Board_Hor(i) == aColumns[j])
      {
        // make sure we add the center columns first
        if (CanPlayAt(i,nColour))
        {
          aMoves->aMoves[nFound].MoveTo = i;
          aMoves->aMoves[nFound].MoveFrom = 0;
          aMoves->aMoves[nFound].bValidMove = true;
          nFound++;
        }
      }
  }

  aMoves->TotalMoves = nFound;
  return nFound;
}

// ---------------------------------------------------------------------------
// execute move and change static board representation
// assume move nPos is valid for colour
// returning true means the move sequence is finished

bool Connect4Eng::ApplyMove(const TSingleMove nPos,
    const int nColour,const int nMoveType)
{
  SetPieceAt(nPos.MoveTo,nColour);

  return true;
}

// ---------------------------------------------------------------------------
// check if putting a stone somewhere is possible

bool Connect4Eng::CanPlayAt(const int nPos,const int nOwnColour)
{
  int nX = Board_Hor(nPos);
  int nY = Board_Ver(nPos);

  if (aBoard[nX][nY] != 0) return false;
  if (nY > 5) return false;
  if (nY > 0) return ( aBoard[nX][nY-1] != 0);

  return true;
}

// ---------------------------------------------------------------------------
// calculate the value of the position

signed long int Connect4Eng::EvaluatePos(const int nColour)
{
  int nX,nY;
  signed long int nValue = 0;

  for (nX=0;nX<7;nX++)
    for (nY=0;nY<6;nY++)
    {
      nValue += EvaluateCase(nX,nY,nColour);
      if (abs(nValue) > SCORE_ISWONPAR) break;
    }

  return nValue;
}

// ---------------------------------------------------------------------------
// calculate the value of the case
// in case it is filled, calculate the static case value,
// in case it is empty, calculate the potential by valuing the
//   combinations possible

signed long int Connect4Eng::EvaluateCase(const int nX,const int nY,const int nColour)
{
  signed long int nValue;

  static int aCaseValues[7][6] = {
    0 , 0 , 1 , 3 , 1 , 0 , 0 ,
    0 , 0 , 1 , 3 , 1 , 0 , 0 ,
    0 , 0 , 2 , 4 , 2 , 0 , 0 ,
    0 , 0 , 2 , 4 , 2 , 0 , 0 ,
    0 , 0 , 1 , 3 , 1 , 0 , 0 ,
    0 , 0 , 1 , 3 , 1 , 0 , 0 };

  signed int aMask[4][7];

  MakeMask(nX-3,nY-3,1, 1,aMask[0]);
  MakeMask(nX-3,nY  ,1, 0,aMask[1]);
  MakeMask(nX-3,nY+3,1,-1,aMask[2]);
  MakeMask(nX  ,nY-3,0, 1,aMask[3]);

  // case value depends on own value minus adversary value
  nValue = EvaluateMask(aMask,nColour) - EvaluateMask(aMask,(3-nColour));

  if (aBoard[nX][nY] == nColour)
    nValue += aCaseValues[nX][nY] ;
  else if (aBoard[nX][nY] == (3-nColour))
    nValue -= aCaseValues[nX][nY] ;

  return nValue; // nColour==1?nValue:-nValue;
}

// ---------------------------------------------------------------------------
// make horizontal representation of stones in a row
// minus one means that there is no case here

void Connect4Eng::MakeMask(int nX,int nY,const int ndX,const int ndY,int aMask[7])
{
  int i;
  for (i=0;i<7;i++)
  {
    if ((nX<0) || (nY<0) || (nX>6) || (nY>5)) aMask[i] = -1;
    else aMask[i] = aBoard[nX][nY];
    nX += ndX;
    nY += ndY;
  }
}

// ---------------------------------------------------------------------------
// evaluate a calculated array of 4 possible directions of a single stone

int Connect4Eng::EvaluateMask(int aMask[4][7],int nColour)
{
  int i,j,nOwn,nInc;

  // test if we have 4
  for (i=0;i<4;i++)
  {
    nOwn = 0;
    for (j=0;j<7;j++)
    {
      if (aMask[i][j] == nColour)
      {
        nOwn++;
        if (nOwn>3)
          return SCORE_ISWON;
      }
      else
        nOwn = 0;
    }
  }

  // test if we have 3
  for (i=0;i<4;i++)
  {
    nOwn = 0;
    for (j=0;j<7;j++)
    {
      if (aMask[i][j] == nColour)
      {
        nOwn++;
        if (nOwn>2)
          return 99;
      }
      else if (aMask[i][j] != 0)
        nOwn = 0;
    }
  }
  return 0;
}

// ---------------------------------------------------------------------------
// check if game is ended

bool Connect4Eng::IsEnded(const int nColour,int *nScoreEnded)
{
  // in case of connect 4, this means the other colour has 4 in a row
  *nScoreEnded = EvaluatePos(nColour);
  if ( abs(*nScoreEnded) > SCORE_ISWONPAR )
    return true;
  else if (GetHistoryIndex()>40)
  // todo: to add the current move in the tree
  {
    *nScoreEnded = SCORE_DRAW;
    return true;
  }
  return false;
}

// ---------------------------------------------------------------------------

int Connect4Eng::MoveRepresentation(TSingleMove *nMove, char *sMove)
{
  return sprintf(sMove,"%i",
                Board_Hor(nMove->MoveTo)+1);
}


// ---------------------------------------------------------------------------
// convert this string back to a move

void Connect4Eng::Str2Move(char *sMove,TSingleMove *nMove)
{
  int i,nCol;

  nCol = sMove[0]-'1';
  for (i=0;i<6;i++)
    if (aBoard[nCol][i] == 0)
      {
        nMove->MoveFrom   = -1;
        nMove->MoveTo     = nCol + i*7;
        nMove->bValidMove = true;
        break;
      }
}

// ---------------------------------------------------------------------------
// eof

