//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "EngDraughts.h"
#include <stdio.h>

#pragma package(smart_init)

//---------------------------------------------------------------------------

static bool bValidCase[] =
                        { true,false,true,false,true,false,true,false,true,false ,
                          false,true,false,true,false,true,false,true,false,true ,
                          true,false,true,false,true,false,true,false,true,false ,
                          false,true,false,true,false,true,false,true,false,true ,
                          true,false,true,false,true,false,true,false,true,false ,
                          false,true,false,true,false,true,false,true,false,true ,
                          true,false,true,false,true,false,true,false,true,false ,
                          false,true,false,true,false,true,false,true,false,true ,
                          true,false,true,false,true,false,true,false,true,false ,
                          false,true,false,true,false,true,false,true,false,true
                        };

//---------------------------------------------------------------------------

__fastcall EngDraughts::EngDraughts(TAlgorithmType nType)
  : EngCheckersBase(nType)
{
  // make sure the priority is high enough to get run before
  // the UI task.  we will call Suspended from the main thread
  // regularly and if the priority is the same as the main thread
  // this would be highly inefficient

  Depth = 6;
  Fields = 100;
}

// ---------------------------------------------------------------------------
// empty the board to its initial status

void EngDraughts::InitBoard(void)
{
  int i;

  EngCheckersBase::InitBoard();
  BoardDimension = 10;

  for (i=0;i<=39;i++)
    if (bValidCase[i]) SetPieceAt(i,1);
  for (i=60;i<=99;i++)
    if (bValidCase[i]) SetPieceAt(i,2);
}

// ---------------------------------------------------------------------------
// check if we could move a stone from a certain position
// and add all possible options to the list of possible moves
// if there is already one capture move, do not add futher
// regular moves to the list

bool EngDraughts::AddValidFrom(const int nPos, const int nOwnColour,
                                  TMoveList *aMoves,const int nMode)
{
  bool bFoundCaptureMove;
  int  nStoneColour = aBoard[Board_Hor(nPos)][Board_Ver(nPos)];

  if ((nStoneColour & 3) != nOwnColour)
    return false; // no own stone here, so no capture move found

  if (nMode == ALL_MOVE_TYPES)
  {
    // search all possible moves, captures and normal ones, once
    // a capture is found, a research will be made entering the other modues
    // so correct housekeeping of the flag for switching modes
    // is not required here
      if (nStoneColour==1)
      {
        // normal white stone
        // allow forward normal moves and backwards captures
        bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,1,1,true);
        bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,-1,1,true) ||
                            bFoundCaptureMove;
        bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,1,-1,false) ||
                            bFoundCaptureMove;
        bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,-1,-1,false) ||
                            bFoundCaptureMove;
      }
      else if (nStoneColour==2)
      {
        // normal black stone
        // allow forward normal moves and backwards captures
        bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,1,-1,true);
        bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,-1,-1,true) ||
                            bFoundCaptureMove;
        bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,1,1,false) ||
                            bFoundCaptureMove;
        bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,-1,1,false) ||
                            bFoundCaptureMove;
      }
      else
      {
        // queen stone
        bFoundCaptureMove = false;
        int i;
        for (i=1;i<10;i++)
        {
          // would require to be 4 different loops in case a research is
          // not done, here irrelevant as all moves will be voided
          // as soon as a capture move is found
          bFoundCaptureMove = AddValidInDirQ(nPos,nOwnColour,aMoves,i,1,1,false) ||
                              bFoundCaptureMove;
          bFoundCaptureMove = AddValidInDirQ(nPos,nOwnColour,aMoves,i,-1,1,false) ||
                              bFoundCaptureMove;
          bFoundCaptureMove = AddValidInDirQ(nPos,nOwnColour,aMoves,i,1,-1,false) ||
                              bFoundCaptureMove;
          bFoundCaptureMove = AddValidInDirQ(nPos,nOwnColour,aMoves,i,-1,-1,false) ||
                              bFoundCaptureMove;
        }
      }
  }
  else
  {
    // question is, do we have to maintain the
    // foundcapturemove variable
    if ((nStoneColour==1) || (nStoneColour==2))
    {
      // allow captures only
      // or condition at end otherwise optimizer removes second
      // evaluation
      // might be irrelevant as the search is redone in case of a capture found
      bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,1,1,false);
      bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,-1,1,false) ||
                          bFoundCaptureMove;
      bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,1,-1,false) ||
                          bFoundCaptureMove;
      bFoundCaptureMove = AddValidInDir(nPos,nOwnColour,aMoves,-1,-1,false) ||
                          bFoundCaptureMove;
    }
    else
    {
      // queen stone
      bFoundCaptureMove = false;  // do we need this housekeeping?
      int i;
      for (i=1;i<10;i++)
      {
        bFoundCaptureMove = AddValidInDirQ(nPos,nOwnColour,aMoves,i,1,1,true) ||
                              bFoundCaptureMove;
        bFoundCaptureMove = AddValidInDirQ(nPos,nOwnColour,aMoves,i,-1,1,true) ||
                              bFoundCaptureMove;
        bFoundCaptureMove = AddValidInDirQ(nPos,nOwnColour,aMoves,i,1,-1,true) ||
                              bFoundCaptureMove;
        bFoundCaptureMove = AddValidInDirQ(nPos,nOwnColour,aMoves,i,-1,-1,true) ||
                              bFoundCaptureMove;
      }
    }
  }

  return bFoundCaptureMove;  // return true for capture move, false for other
}

// ---------------------------------------------------------------------------
// add move from pos in direction to movelist
// will possibly convert from allmovetypes to captures only
// by returning the captures only flag

bool inline EngDraughts::AddValidInDir(const int nPos,
      const int nOwnColour, TMoveList * aMoves,
      const int nDirHor, const int nDirVer, const bool bAllMoveTypes)
{
  int bFoundCaptureMove = false;
  int nHor = Board_Hor(nPos);
  int nVer = Board_Ver(nPos);
  int nNewVer = nVer+nDirVer;
  int nNewHor = nHor+nDirHor;

  // only one test is needed but we do not know if we are going forward
  // or backward, so just test both

  if ((nNewVer >= 0) && (nNewVer<BoardDimension)
       && (nNewHor >= 0) && (nNewHor<BoardDimension))
  {
    if (aBoard[nNewHor][nNewVer]==0)
    {
      // empty case, so normal move is possible
      if (bAllMoveTypes)
        // we are seeking for all moves, not only
        // just the capture moves
        AddValidMoveToList(aMoves,nPos,nNewHor+nNewVer*BoardDimension,false);
    }
    else if ((aBoard[nNewHor][nNewVer]&3)!=nOwnColour)
    {
      nNewVer += nDirVer;
      nNewHor += nDirHor;
      if ((nNewVer >= 0) && (nNewVer<BoardDimension)
          && (nNewHor >= 0) && (nNewHor<BoardDimension))
      {
        // we are still on the board, so the move is valid
        if (aBoard[nNewHor][nNewVer]==0)
        {
          // and square we want to jump too is empty
          // so store this capture move
          AddValidMoveToList(aMoves,nPos,(nNewHor)+(nNewVer)*BoardDimension,true);
          bFoundCaptureMove = true;
        }
      }
    }
  }
  return bFoundCaptureMove;
}

// ---------------------------------------------------------------------------
// we are considering queen moves only
// add move from pos to indicated field to movelist
// will check if there is a queen jump required
// will possibly convert from allmovetypes to captures only
// by returning the captures only flag

bool inline EngDraughts::AddValidInDirQ(const int nPos,
      const int nOwnColour, TMoveList * aMoves,const int nDistance,
      const int nDirHor, const int nDirVer,const bool bCaptureMode)
{
  int bFoundCaptureMove = false;
  int nHor = Board_Hor(nPos);
  int nVer = Board_Ver(nPos);
  int nNewVer = nVer+nDirVer*nDistance;
  int nNewHor = nHor+nDirHor*nDistance;

  // only one test is needed but we do not know if we are going forward
  // or backward, so just test both

  if ((nNewVer >= 0) && (nNewVer<BoardDimension)
       && (nNewHor >= 0) && (nNewHor<BoardDimension))
  {
    if (aBoard[nNewHor][nNewVer]==0)
    {
      // empty case, so move seems possible
      // check if any of the preceeding cases is just one stone, if more,
      // this is not a valid move, if just one, this is a capture move
      // if none, this is a normal move, if own stone, no valid move either
      int nAdvStone = 0;
      { // scope variables
        int nStone;
        int i=0;
        while(++i<nDistance)
        {
          // if nDistance is 1, no search is required
          nStone = aBoard[nHor+nDirHor*i][nVer+nDirVer*i]&3;
          if (nStone==nOwnColour) nAdvStone=2; // this is invalid
          else if (nStone==(3-nOwnColour)) nAdvStone++;
        }
      }

      if (nAdvStone==0)
      {
        // normal move
        if (!bCaptureMode)
          // in case in capture mode only, we do not allow regular moves
          AddValidMoveToList(aMoves,nPos,nNewHor+nNewVer*BoardDimension,false);
      }
      else if (nAdvStone == 1)
      {
        // capture move
        AddValidMoveToList(aMoves,nPos,nNewHor+nNewVer*BoardDimension,true);
        bFoundCaptureMove = true;
      }
    }
  }
  return bFoundCaptureMove;
}

// ---------------------------------------------------------------------------
// execute move and change static board representation
// assume move nPos is valid for colour
// return true if a possible next capture can be done

bool EngDraughts::ApplyMove(const TSingleMove nPos,const int nColour,
    const int nMoveType)
{
  TSingleMove nMove;
  int nDiffVer,nDiffHor,nDist;

  SetPieceAt(nPos.MoveTo,GetPieceAt(nPos.MoveFrom));
  SetPieceAt(nPos.MoveFrom,0);

  nDiffVer = Board_Ver(nPos.MoveTo)-Board_Ver(nPos.MoveFrom);
  nDist = abs(nDiffVer);

  if (nDist > 1)
  {
    // this is a capture move, make sure to remove the piece that was captured
    // the captured piece is one ahead of the vertical location and is in
    // the middle of the old and new horizontal position

    // normalize the differences in -1 or +1, the multiplier is held in nDist
    nDiffHor = (Board_Hor(nPos.MoveTo)-Board_Hor(nPos.MoveFrom)) / nDist;
    nDiffVer = nDiffVer / nDist;

    for (int i=1;i<nDist;i++)
    {
      // now clear out all pieces in between the originating
      // piece and landing piece
      // for this we need to have the increment both
      // horizontally and vertically
      SetPieceAt(  ( Board_Hor(nPos.MoveFrom) + i * nDiffHor)
                  +( Board_Ver(nPos.MoveFrom) + i * nDiffVer)
                    *BoardDimension,0);

    }

    if (nMoveType == ALL_MOVE_TYPES)
    {
      // do iterate from here
      nMove = nPos;
      while (ApplyNextMove(&nMove,nColour));
    }

    return true;
  }

  // no capture move found
  // do promotion
  if ((nColour==1)&&(Board_Ver(nPos.MoveTo)==(BoardDimension-1)))
    SetPieceAt(nPos.MoveTo,5);
  if ((nColour==2)&&(Board_Ver(nPos.MoveTo)==0))
    SetPieceAt(nPos.MoveTo,6);

  return false;
}

// ---------------------------------------------------------------------------
// calculate the value of the position

signed long int EngDraughts::EvaluatePos(const int nColour)
{
  register int i,j;

  signed long int liValue = 0;
  signed long int aValues[] = { 0 , 10 , -10 , 0 , 0 , 30 , -30 };
  // allow space for normal pieces and for queens

  for (i=0;i<BoardDimension;i++)
    for (j=0;j<BoardDimension;j++)
      if (aBoard[i][j])
        liValue += (aValues[aBoard[i][j]]);

  return nColour==1?liValue:-liValue;
}

// ---------------------------------------------------------------------------
// eof






