//---------------------------------------------------------------------------

#ifndef GameEngH
#define GameEngH
//---------------------------------------------------------------------------

#include <string>
#include <Classes.hpp>

#include "GameClasses.h"

class Algorithm;

enum TAlgorithmType {ALG_RANDOM,
                     ALG_STATIC_EVAL,
                     ALG_BRUTE_FORCE,
                     ALG_BRUTE_FWIN,
                     ALG_ALPHA_BETA,
                     ALG_NEGASCOUT};

//---------------------------------------------------------------------------
class GameEngine : public TThread
{
private:
  int FDepth;
  TSingleMove aHistory[MAX_MOVES_IN_HISTORY];
  int nCurrentMoveInHistory;

protected:
  int aBoard[MAX_BOARD_DIMENSION][MAX_BOARD_DIMENSION];
  int nToMove;
  int Fields;
  int BoardDimension;

  void __fastcall SetDepth(const int value);
  int __fastcall GetDepth();
  void __fastcall Execute();

  virtual bool PlayerHasMove(const int nColour);
  virtual void InitBoard(void);

  // for easy calculation of positions
  virtual int Board_Hor(const int nPos);
  virtual int Board_Ver(const int nPos);

public:
  TSingleMove nSelectedMove;
  int MaxUniqueMoves;
  int nVariant;
  Algorithm *Search;
  long int nNodes;

  // different search algorithms
  friend class Algorithm;

  __fastcall GameEngine(TAlgorithmType nType);
  void SetAlgorithm(TAlgorithmType nType);
  void LogBoard();

  // generic entries
  virtual TSingleMove CalcMove(void);
  virtual bool IsValidMove(TSingleMove& nPos,const int nColour);
  void SetPlayer(const int nColour);
  void SetPieceAt(const int nPos,const int nColour);
  int  GetPieceAt(const int nPos);

  // move history handling
  int  GetHistoryIndex(void);
  void GetHistoryMove(const int nIndex,TSingleMove *nMove);
  void SetHistoryMove(TSingleMove& nMove);
  bool ClearHistoryMove(void);
  virtual int MoveRepresentation(TSingleMove *nMove, char *sMove);
  virtual void Str2Move(char *sMove,TSingleMove *nMove) = 0;

  virtual int GenerateMoves(TMoveList *aMoves,const int nColour) = 0;
  virtual bool IsEnded(const int nColour,int *nScoreEnded);
  virtual bool IsValidFrom(const int nPos,const int nColour);
  virtual signed long int EvaluatePos(const int nColour);

  virtual void PushBoard(TBoard *aBoard);
  virtual void PopBoard(const TBoard aBoard);

  // interface for UI code
  virtual bool ApplyMove(TSingleMove nMove,const int nColour,
      const int nMoveType) = 0;
  virtual bool ApplyNextMove(TSingleMove *nMove,const int nColour);
  __fastcall ~GameEngine();

__published:
  __property int Depth  = { read=GetDepth, write=SetDepth };
};
//---------------------------------------------------------------------------
#endif
