//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "EngTicTacToe.h"
#include <stdio.h>
#pragma package(smart_init)

//---------------------------------------------------------------------------

__fastcall EngTicTacToe::EngTicTacToe(TAlgorithmType nType)
  : GameEngine(nType)
{
  // make sure the priority is high enough to get run before
  // the UI task.  we will call Suspended from the main thread
  // regularly and if the priority is the same as the main thread
  // this would be highly inefficient

  Depth = 9;
  Fields = 9;
  MaxUniqueMoves = 9;
  BoardDimension = 3;
}

// ---------------------------------------------------------------------------
// empty the board to its initial status

void EngTicTacToe::InitBoard(void)
{
  int i;

  GameEngine::InitBoard();
  BoardDimension = 3;
}
// ---------------------------------------------------------------------------
// generate a list of valid moves

int EngTicTacToe::GenerateMoves(TMoveList *aMoves,const int nColour)
{
  register int i;
  int nFound = 0;

  // see if can take and add to list of valid moves
  for (i=0;i<9;i++)
  {
    // make sure we add the center columns first
    if (CanPlayAt(i,nColour))
    {
      aMoves->aMoves[nFound].MoveTo = i;
      aMoves->aMoves[nFound].MoveFrom = 0;
      aMoves->aMoves[nFound].bValidMove = true;
      nFound++;
    }
  }

  aMoves->TotalMoves = nFound;
  return nFound;
}

// ---------------------------------------------------------------------------
// execute move and change static board representation
// assume move nPos is valid for colour
// returning true means the move sequence is finished

bool EngTicTacToe::ApplyMove(const TSingleMove nPos,
    const int nColour,const int nMoveType)
{
  SetPieceAt(nPos.MoveTo,nColour);

  return true;
}

// ---------------------------------------------------------------------------
// check if putting a stone somewhere is possible

bool EngTicTacToe::CanPlayAt(const int nPos,const int nOwnColour)
{
  int nX = Board_Hor(nPos);
  int nY = Board_Ver(nPos);

  if (aBoard[nX][nY] != 0) return false;

  return true;
}

// ---------------------------------------------------------------------------
// calculate the value of the position

signed long int EngTicTacToe::EvaluatePos(const int nColour)
{
  for (int i=0;i<3;i++)
  {
    // vertical
    if ((aBoard[i][0] == aBoard[i][1]) &&
        (aBoard[i][1] == aBoard[i][2]) &&
        (aBoard[i][2] == nColour))
        return SCORE_ISWON;

    // horizontal
    if ((aBoard[0][i] == aBoard[1][i]) &&
        (aBoard[1][i] == aBoard[2][i]) &&
        (aBoard[2][i] == nColour))
        return SCORE_ISWON;
  }

  // diagonal up
  if ((aBoard[0][0] == aBoard[1][1]) &&
      (aBoard[1][1] == aBoard[2][2]) &&
      (aBoard[2][2] == nColour))
      return SCORE_ISWON;

  // diagonal down
  if ((aBoard[0][2] == aBoard[1][1]) &&
      (aBoard[1][1] == aBoard[2][0]) &&
      (aBoard[2][0] == nColour))
      return SCORE_ISWON;

  return SCORE_DRAW;
}

// ---------------------------------------------------------------------------
// check if game is ended

bool EngTicTacToe::IsEnded(const int nColour,int *nScoreEnded)
{
/*
  *nScoreEnded = -EvaluatePos(nColour);
  if ( abs(*nScoreEnded) > SCORE_ISWONPAR )
    return true;
*/
  // as this function is called after a switchmove, this is more
  // relevant to test
  *nScoreEnded = EvaluatePos(nColour);
  if ( *nScoreEnded > SCORE_ISWONPAR )
    return true;

  for (int i=0;i<9;i++)
    if (aBoard[Board_Hor(i)][Board_Ver(i)] == 0) return false;

  *nScoreEnded = SCORE_DRAW;
  return true;
}

// ---------------------------------------------------------------------------

int EngTicTacToe::MoveRepresentation(TSingleMove *nMove, char *sMove)
{
  sprintf(sMove,"%c%c",
          (char)('a'+(char)Board_Hor(nMove->MoveTo)),
          (char)('0'+(char)Board_Ver(nMove->MoveTo)+1));
  return 0;
}


// ---------------------------------------------------------------------------
// convert this string back to a move

void EngTicTacToe::Str2Move(char *sMove,TSingleMove *nMove)
{
  nMove->MoveFrom   = -1;
  nMove->MoveTo     = (sMove[0]-'a') + (sMove[1]-'1')*BoardDimension;
  nMove->bValidMove = true;
}

// ---------------------------------------------------------------------------
// eof

