//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "MainUI.h"
#include "BoardDraughtsUI.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "BoardPiece"
#pragma resource "*.dfm"
TDraughtsFrame *DraughtsFrame;
//---------------------------------------------------------------------------
__fastcall TDraughtsFrame::TDraughtsFrame(TComponent* Owner)
  : TFrame(Owner)
{
  int i;
  String sPath = ExtractFilePath(ParamStr(0));

  for (i=0;i<NUM_DRAUGHTSSTONES;i++)
    pPiece[i] = new Graphics::TBitmap;

  pPiece[0]->LoadFromFile(sPath+"Images\\Checkers_WhiteStone.bmp");
  pPiece[1]->LoadFromFile(sPath+"Images\\Checkers_BlackStone.bmp");
  pPiece[2]->LoadFromFile(sPath+"Images\\Checkers_WhiteQueen.bmp");
  pPiece[3]->LoadFromFile(sPath+"Images\\Checkers_BlackQueen.bmp");
  pPiece[4]->LoadFromFile(sPath+"Images\\Checkers_EmptyStone.bmp");

  // make sure no single non playable field does contain a tag
  // value that is used for finding pieces
  for (i=0;i<DraughtsPanel->ControlCount;i++)
  {
    if (dynamic_cast <TShape*> (DraughtsPanel->Controls[i]))
    {
      ((TShape*)DraughtsPanel->Controls[i])->Tag = -1;
    }
    else if (dynamic_cast <TBoardPiece*> (DraughtsPanel->Controls[i]))
    {
      ((TBoardPiece*)DraughtsPanel->Controls[i])->SetPicture(0,pPiece[0]);
      ((TBoardPiece*)DraughtsPanel->Controls[i])->SetPicture(1,pPiece[1]);
      ((TBoardPiece*)DraughtsPanel->Controls[i])->SetPicture(2,pPiece[2]);
      ((TBoardPiece*)DraughtsPanel->Controls[i])->SetPicture(3,pPiece[3]);
      ((TBoardPiece*)DraughtsPanel->Controls[i])->SetPicture(4,pPiece[4]);
    }
  }

  nSelected = -1;
  
#ifdef _DEBUG
  // update the hints display so we can esily debug
  // we might need to remove this setting for final version
  for (i=0;i<DraughtsPanel->ControlCount;i++)
    if (dynamic_cast <TBoardPiece*> (DraughtsPanel->Controls[i]))
      ((TBoardPiece*)DraughtsPanel->Controls[i])->Hint =
        ((TBoardPiece*)DraughtsPanel->Controls[i])->Tag;
#endif
}
//---------------------------------------------------------------------------

int TDraughtsFrame::GetControlCount(void)
{
  return DraughtsPanel->ControlCount;
}

//---------------------------------------------------------------------------

int TDraughtsFrame::GetObjectValue(int nIndex)
{
  return DraughtsPanel->Controls[nIndex]->Tag;
}

//---------------------------------------------------------------------------

void TDraughtsFrame::SetPieceAt(int nPos, int nColour)
{
  int i, nMax;
  int aMapper [] = { 4 , 0 , 1 , -1 , -1 , 2 , 3 };
  int iPicture = aMapper[nColour];

  nMax = DraughtsPanel->ControlCount;

  for (i=0;i<nMax;i++)
  {
    if (dynamic_cast <TBoardPiece*> (DraughtsPanel->Controls[i]))
    {
      if (nPos == GetObjectValue(i))
      {
        if (((TBoardPiece*)(DraughtsPanel->Controls[i]))->GetPictureID() !=
            iPicture)
          ((TBoardPiece*)(DraughtsPanel->Controls[i]))->ActivatePicture(iPicture);
        return;
      }
    }
  }
}

//---------------------------------------------------------------------------

void __fastcall TDraughtsFrame::BoardPieceMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
  if (nSelected==-1)
  {
    if (MainForm->IsValidFrom(((TBoardPiece*)Sender)->Tag))
    {
      // possible to move from
      ClearSelects();
      ((TBoardPiece*)Sender)->Transparent = false;
    }
  }
  else
  {
    // new selection, so try move
    TSingleMove nMove;

    nMove.MoveFrom = nSelected;
    nMove.MoveTo = ((TBoardPiece*)Sender)->Tag;
    if (MainForm->IsValidMove(nMove))
      // possible to execute this move
      ((TBoardPiece*)Sender)->Transparent = false;
  }
}

//---------------------------------------------------------------------------
// clear all selected cases on the board

void TDraughtsFrame::ClearSelects(void)
{
  int i;

  for (i=0;i<DraughtsPanel->ControlCount;i++)
    if (dynamic_cast <TBoardPiece*> (DraughtsPanel->Controls[i]))
      if (!((TBoardPiece*)DraughtsPanel->Controls[i])->Transparent)
        // this test avoids flickering by only updating when there is something
        // to update
        ((TBoardPiece*)DraughtsPanel->Controls[i])->Transparent = true;

  nSelected = -1;
}
//---------------------------------------------------------------------------

void __fastcall TDraughtsFrame::BoardPieceClick(TObject *Sender)
{
  if (nSelected==-1)
  {
    // no field selected, so apply move
    if (MainForm->IsValidFrom(((TBoardPiece*)Sender)->Tag))
    {
      ClearSelects();
      ((TBoardPiece*)Sender)->Transparent = false;
      nSelected = ((TBoardPiece*)Sender)->Tag;
    }
  }
  else
  {
    // multi-mode, first entry is select
    // second entry is a new select or a move
    if (nSelected == ((TBoardPiece*)Sender)->Tag)
    {
      // if twice the same field is selected, the move is undone
      ClearSelects();
      // and at the same time reset the colour as in mousemove
      // will avoid flickering
      ((TBoardPiece*)Sender)->Transparent = true;
    }
    else
    {
      // new selection, so try move
      ((TBoardPiece*)Sender)->Transparent = false;

      TSingleMove nMove;

      nMove.MoveFrom = nSelected;
      nMove.MoveTo = ((TBoardPiece*)Sender)->Tag;
      /*
      if (abs(Board_Hor(nMove.MoveTo)-Board_Hor(nMove.MoveFrom))
           != 1)
        nMove.bCaptureMove = true;
      */
      
      if (MainForm->IsValidMove(nMove))
      {
        // move done
        MainForm->PlayValidMove(nMove);
        ClearSelects();
      }
      else
      {
        // move not possible, undo display
        // we use the empty colour just to remove the border of the stone
        ((TBoardPiece*)Sender)->Transparent = true;
        // and redisplay new starting position if a possible move
        // found
        if (MainForm->IsValidFrom(((TBoardPiece*)Sender)->Tag))
        {
          ClearSelects();
          ((TBoardPiece*)Sender)->Transparent = false;
          nSelected = ((TBoardPiece*)Sender)->Tag;
        }
      }
    }
  }
}

//---------------------------------------------------------------------------
// eof


