//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Connect4BoardUI.h"
#include "MainUI.h"

extern GameUIData *mGameUI;

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "BoardPiece"
#pragma resource "*.dfm"
TConnect4BoardFrame *Connect4BoardFrame;
//---------------------------------------------------------------------------
__fastcall TConnect4BoardFrame::TConnect4BoardFrame(TComponent* Owner)
  : TFrame(Owner)
{
  int i;
  String sPath = ExtractFilePath(ParamStr(0));

  for (i=0;i<3;i++)
    pPiece[i] = new Graphics::TBitmap;

  pPiece[0]->LoadFromFile(sPath+"Images\\Connect4_EmptyStone.bmp");
  pPiece[1]->LoadFromFile(sPath+"Images\\Connect4_WhiteStone.bmp");
  pPiece[2]->LoadFromFile(sPath+"Images\\Connect4_BlackStone.bmp");

  nSelected = -1;

  for (i=0;i<ControlCount;i++)
  {
    if (dynamic_cast <TBoardPiece*> (Controls[i]))
    {
      ((TBoardPiece*)Controls[i])->SetPicture(0,pPiece[0]);
      ((TBoardPiece*)Controls[i])->SetPicture(1,pPiece[1]);
      ((TBoardPiece*)Controls[i])->SetPicture(2,pPiece[2]);
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TConnect4BoardFrame::PieceMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
  TSingleMove nMove;

  nMove.MoveFrom = 0;
  nMove.MoveTo = ((TBoardPiece*)Sender)->Tag;

  if (MainForm->IsValidMove(nMove))
  {
    // if there was another move selected before
    if ((nSelected != -1) && (nSelected != (long) nMove.MoveTo))
      ClearSelects();

    ((TBoardPiece*)Sender)->Transparent = false;
    nSelected = nMove.MoveTo;
  }
  else
  {
    // no valid move under the stone, so clear the selected move
    ClearSelects();
  }
}
//---------------------------------------------------------------------------
void __fastcall TConnect4BoardFrame::PieceMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
  TSingleMove nMove;

  nMove.MoveFrom = 0;
  nMove.MoveTo = ((TBoardPiece*)Sender)->Tag;

  if (MainForm->IsValidMove(nMove))
  {
    ClearSelects();
    MainForm->PlayValidMove(nMove);
  }
}

//---------------------------------------------------------------------------
// clear all selected moves on the board

void TConnect4BoardFrame::ClearSelects(void)
{
  int i;

  for (i=0;i<ControlCount;i++)
    if (dynamic_cast <TBoardPiece*> (Controls[i]))
      if (!((TBoardPiece*)Controls[i])->Transparent)
        // this test avoids flickering by only updating when there is something
        // to update
        ((TBoardPiece*)Controls[i])->Transparent = true;

  nSelected = -1;
}

//---------------------------------------------------------------------------

void TConnect4BoardFrame::SetPieceAt(int nPos, int nColour)
{
  int i, nMax;

  nMax = ControlCount;

  for (i=0;i<nMax;i++)
  {
    if (nPos == GetObjectValue(i))
    {
      if (((TBoardPiece*)Controls[i])->GetPictureID() != nColour)
        ((TBoardPiece*)Controls[i])->ActivatePicture(nColour);
      return;
    }
  }
}

//---------------------------------------------------------------------------

int TConnect4BoardFrame::GetControlCount(void)
{
  return ControlCount;
}

//---------------------------------------------------------------------------

int TConnect4BoardFrame::GetObjectValue(int nIndex)
{
  return Controls[nIndex]->Tag;
}

//---------------------------------------------------------------------------
// eof

