//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "StrategicGames.h"
#include "MainUI.h"
#include "AboutUI.h"
#include "SelectGameUI.h"
#include "GameUICollection.h"
#include "DocumentationUI.h"

extern GameUIData *mGameUI;
extern TGame      *mGameData;

//---------------------------------------------------------------------------
#pragma link "EngineUI"
#pragma link "OthelloBoardUI"
#pragma link "Connect4BoardUI"
#pragma link "CheckersBoardUI"
#pragma link "Connect4BoardUI"
#pragma link "EngineUI"
#pragma link "OthelloBoardUI"
#pragma link "BoardPiece"
#pragma link "BoardDraughtsUI"
#pragma link "BoardTicTacToeUI"
#pragma resource "*.dfm"
TMainForm *MainForm;

//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
	: TForm(Owner)
{
  int i;

  bFirstRun = true;

  for (i=0;i<2;i++)
    pPiece[i] = new Graphics::TBitmap;

  // mGameData->nComputerColour = 2;
  // set in mGameData constructor
}

//----------------------------------------------------------------------------

__fastcall TMainForm::~TMainForm()
{
  // only delete it here as not to loose references to
  // the bitmap in memory while running
  // delete pPiece[0];
  // delete pPiece[1];
}

//----------------------------------------------------------------------------

void __fastcall TMainForm::FormCreate(TObject *Sender)
{
	Application->OnHint = ShowHint;
  Caption = Application->Title;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
  delete mGameUI;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::ShowHint(TObject *Sender)
{
	StatusLine->SimpleText = Application->Hint;
}

//----------------------------------------------------------------------------
void __fastcall TMainForm::FileSave(TObject *Sender)
{
  FrameEngineUI->SaveGame(ExtractFilePath(ParamStr(0))+
            mGameUI->sName + ".pgn");
}

//----------------------------------------------------------------------------
void __fastcall TMainForm::OpenActionExecute(TObject *Sender)
{
  if (FrameEngineUI->ResetDisAllowed()) return;

  FrameEngineUI->OpenGame(ExtractFilePath(ParamStr(0))+
            mGameUI->sName + ".pgn");
}

//---------------------------------------------------------------------------
void __fastcall TMainForm::FileExit(TObject *Sender)
{
	Close();
}
//----------------------------------------------------------------------------

void __fastcall TMainForm::HelpRulesItemClick(TObject *Sender)
{
  DocumentationForm->IE->Navigate(
    WideString(ExtractFilePath(ParamStr(0))+"rules.html"+
    "#"+mGameUI->sName));
  DocumentationForm->ShowModal();
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::DocumentationItemClick(TObject *Sender)
{
  DocumentationForm->IE->Navigate(
    WideString(ExtractFilePath(ParamStr(0))+"readme.html"));
  DocumentationForm->ShowModal();
}

//----------------------------------------------------------------------------

void __fastcall TMainForm::HelpAbout(TObject *Sender)
{
	AboutBox->ShowModal();
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
// number of routines to start a new game
// menu items first

void __fastcall TMainForm::FileNew(TObject *Sender)
{
  if (StartAGame())
  {
    StartGivenGameUI();
    StartGivenGame();
  }
}

//---------------------------------------------------------------------------
// let the user select a game, if the user does not select any
// game, return false

bool TMainForm::StartAGame()
{
  if (FrameEngineUI->ResetDisAllowed()) return false;

  SelectGameForm->ShowModal();

  delete mGameUI;

  if (SelectGameForm->mSelectedGame==GAMETYPE_OTHELLO)
    mGameUI = (GameUIData*) new OthelloUIData();
  else if (SelectGameForm->mSelectedGame==GAMETYPE_CONNECT4)
    mGameUI = (GameUIData*) new Connect4UIData();
  else if (SelectGameForm->mSelectedGame==GAMETYPE_TICTACTOE)
    mGameUI = (GameUIData*) new TicTacToeUIData();
  else if (SelectGameForm->mSelectedGame==GAMETYPE_CHECKERS_AMERICAN)
    mGameUI = (GameUIData*) new CheckersUIData();
  else if (SelectGameForm->mSelectedGame==GAMETYPE_DRAUGHTS)
    mGameUI = (GameUIData*) new DraughtsUIData();
  else
  {
    // one has to be the default, ....
    mGameUI = (GameUIData*) new CheckersUIData();
    mGameUI->mGameType=GAMETYPE_CHECKERS_SIMPLE;
  }

  return true;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::NewOthelloActionExecute(TObject *Sender)
{
  if (FrameEngineUI->ResetDisAllowed()) return;

  delete mGameUI;
  mGameUI = (GameUIData*) new OthelloUIData();
  StartGivenGameUI();
  StartGivenGame();
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::NewACheckersActionExecute(TObject *Sender)
{
  if (FrameEngineUI->ResetDisAllowed()) return;

  delete mGameUI;
  mGameUI = (GameUIData*) new CheckersUIData();
  StartGivenGameUI();
  StartGivenGame();
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::NewSCheckersActionExecute(TObject *Sender)
{
  if (FrameEngineUI->ResetDisAllowed()) return;

  delete mGameUI;
  mGameUI = (GameUIData*) new CheckersUIData();
  mGameUI->mGameType=GAMETYPE_CHECKERS_SIMPLE;
  StartGivenGameUI();
  StartGivenGame();
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::NewDraughtsActionExecute(TObject *Sender)
{
  if (FrameEngineUI->ResetDisAllowed()) return;

  delete mGameUI;
  mGameUI = (GameUIData*) new DraughtsUIData();
  StartGivenGameUI();
  StartGivenGame();
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::NewConnect4ActionExecute(TObject *Sender)
{
  if (FrameEngineUI->ResetDisAllowed()) return;

  delete mGameUI;
  mGameUI = (GameUIData*) new Connect4UIData();
  StartGivenGameUI();
  StartGivenGame();
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::NewTicTacToeActionExecute(TObject *Sender)
{
  if (FrameEngineUI->ResetDisAllowed()) return;

  delete mGameUI;
  mGameUI = (GameUIData*) new TicTacToeUIData();
  StartGivenGameUI();
  StartGivenGame();
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::NewSameButtonClick(TObject *Sender)
{
  if (FrameEngineUI->ResetDisAllowed()) return;

  // StartGivenGameUI();  // not required as screen is right
  StartGivenGame();
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
// core routine called when a game type is being selected

void TMainForm::StartGivenGame(void)
{
  mGameData->init();

  // todo: this kind should be the sort of implementation
  // that is required
  //(castedBoardFrame)->GetProcByName("ClearSelects()");

  if (mGameUI->mGameType==GAMETYPE_OTHELLO)
    OthelloBoardFrame->ClearSelects();
  else if (mGameUI->mGameType==GAMETYPE_CONNECT4)
    Connect4BoardFrame->ClearSelects();
  else
    CheckersBoardFrame->ClearSelects();

  ToMoveShape->Shape = mGameUI->mPieceShape;
  FrameEngineUI->NewGame();
}

//---------------------------------------------------------------------------
// change all screen parameters to allow a certain game
// to be started

void TMainForm::StartGivenGameUI()
{
  String sPath = ExtractFilePath(ParamStr(0));

  // turn all switchable items off and the turn specifics back on
  OthelloBoardFrame->Visible = false;
  CheckersBoardFrame->Visible = false;
  Connect4BoardFrame->Visible = false;
  DraughtsFrame->Visible = false;
  TicTacToeFrame->Visible = false;
  ToMoveShape->Visible = false;
  ToMoveBoardPiece->Visible = false;

  if (mGameUI->mGameType==GAMETYPE_OTHELLO)
  {
    OthelloBoardFrame->Visible = true;
    ToMoveShape->Visible = true;
  }
  else if (mGameUI->mGameType==GAMETYPE_CONNECT4)
  {
    Connect4BoardFrame->Visible = true;
    ToMoveShape->Visible = true;
  }
  else if (mGameUI->mGameType==GAMETYPE_TICTACTOE)
  {
    pPiece[0]->LoadFromFile(sPath+"Images\\TicTacToe_WhiteStone.bmp");
    pPiece[1]->LoadFromFile(sPath+"Images\\TicTacToe_BlackStone.bmp");
    ToMoveBoardPiece->SetPicture(0,pPiece[0]);
    ToMoveBoardPiece->SetPicture(1,pPiece[1]);
    TicTacToeFrame->Visible = true;
    ToMoveBoardPiece->Visible = true;
  }
  else if (mGameUI->mGameType==GAMETYPE_DRAUGHTS)
  {
    pPiece[0]->LoadFromFile(sPath+"Images\\Checkers_WhiteStone.bmp");
    pPiece[1]->LoadFromFile(sPath+"Images\\Checkers_BlackStone.bmp");
    ToMoveBoardPiece->SetPicture(0,pPiece[0]);
    ToMoveBoardPiece->SetPicture(1,pPiece[1]);
    DraughtsFrame->Visible = true;
    ToMoveBoardPiece->Visible = true;
  }
  else
  {
    pPiece[0]->LoadFromFile(sPath+"Images\\Checkers_WhiteStone.bmp");
    pPiece[1]->LoadFromFile(sPath+"Images\\Checkers_BlackStone.bmp");
    ToMoveBoardPiece->SetPicture(0,pPiece[0]);
    ToMoveBoardPiece->SetPicture(1,pPiece[1]);
    CheckersBoardFrame->Visible = true;
    ToMoveBoardPiece->Visible = true;
  }

  Caption = Application->Title + " " + mGameUI->sName;
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
// read the number of boardpositions

int TMainForm::GetBoardPositions(void)
{
  if (mGameUI->mGameType==GAMETYPE_OTHELLO)
    return OthelloBoardFrame->GetControlCount();
  else if (mGameUI->mGameType==GAMETYPE_CONNECT4)
    return Connect4BoardFrame->GetControlCount();
  else if (mGameUI->mGameType==GAMETYPE_TICTACTOE)
    return TicTacToeFrame->GetControlCount();
  else if (mGameUI->mGameType==GAMETYPE_DRAUGHTS)
    return DraughtsFrame->GetControlCount();
  else
    return CheckersBoardFrame->GetControlCount();
}

//---------------------------------------------------------------------------
// get the tagvalue of the object on a certain index

int TMainForm::GetObjectValue(int nPos)
{
  if (mGameUI->mGameType==GAMETYPE_OTHELLO)
    return OthelloBoardFrame->GetObjectValue(nPos);
  else if (mGameUI->mGameType==GAMETYPE_CONNECT4)
    return Connect4BoardFrame->GetObjectValue(nPos);
  else if (mGameUI->mGameType==GAMETYPE_TICTACTOE)
    return TicTacToeFrame->GetObjectValue(nPos);
  else if (mGameUI->mGameType==GAMETYPE_DRAUGHTS)
    return DraughtsFrame->GetObjectValue(nPos);
  else
    return CheckersBoardFrame->GetObjectValue(nPos);
}

//---------------------------------------------------------------------------
// convert engine data to screen data

void TMainForm::UpdateBoard(void)
{
  int i;
  int nPiece;
  int nPos;

  int nMax = GetBoardPositions();

  for (i=0;i<nMax;i++)
  {
    nPos = GetObjectValue(i);
    if (nPos>=0)
    {
      nPiece = FrameEngineUI->GetStoneAt(nPos);
      UISetPieceAt(nPos,nPiece);
    }
  }
}

//---------------------------------------------------------------------------
// just refresh the screen

void TMainForm::UpdateDisplay(void)
{
  ComputerStartsAction->Enabled = (mGameData->nGameStatus == GAMESTATUS_INIT);

  UpdateBoard();

  if (mGameData->nToMove == 1)
  {
    ToMoveShape->Brush->Color = mGameUI->mColourWhite;
    ToMoveBoardPiece->ActivatePicture(0);
  }
  else
  {
    ToMoveShape->Brush->Color = mGameUI->mColourBlack;
    ToMoveBoardPiece->ActivatePicture(1);
  }
}

//---------------------------------------------------------------------------

void TMainForm::UISetPieceAt(int nPos, int nPiece)
{
  if (mGameUI->mGameType==GAMETYPE_OTHELLO)
    OthelloBoardFrame->SetPieceAt(nPos,nPiece);
  else if (mGameUI->mGameType==GAMETYPE_CONNECT4)
    Connect4BoardFrame->SetPieceAt(nPos,nPiece);
  else if (mGameUI->mGameType==GAMETYPE_TICTACTOE)
    TicTacToeFrame->SetPieceAt(nPos,nPiece);
  else if (mGameUI->mGameType==GAMETYPE_DRAUGHTS)
    DraughtsFrame->SetPieceAt(nPos,nPiece);
  else
    CheckersBoardFrame->SetPieceAt(nPos,nPiece);
}

//---------------------------------------------------------------------------
// perform move without first checking if the move is valid

void TMainForm::PlayValidMove(TSingleMove nMove)
{
  FrameEngineUI->PlayValidMove(nMove);
  FrameEngineUI->SwitchUser();
}

//---------------------------------------------------------------------------
// return true if a cetain move can be played

bool TMainForm::IsValidMove(TSingleMove nMove)
{
  return (FrameEngineUI->IsValidMove(nMove));
}

//---------------------------------------------------------------------------
// return true if from case can be used

bool TMainForm::IsValidFrom(int nPos)
{
  return (FrameEngineUI->IsValidFrom(nPos,mGameData->nToMove));
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::FormActivate(TObject *Sender)
{
  if (bFirstRun)
  {
    bFirstRun = false;
    HelpAbout(Sender);
    while (!StartAGame());  // loop until the user select a game
    StartGivenGameUI();
    StartGivenGame();
  }
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::FrameEngineUIComboBoxLevelChange(
      TObject *Sender)
{
  FrameEngineUI->ComboBoxLevelChange(Sender);
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::ComputerStartsActionExecute(TObject *Sender)
{
  mGameData->nToMove = 2;
  mGameData->nComputerColour = 1;
  FrameEngineUI->SwitchUser();
}

//---------------------------------------------------------------------------
// eof



