//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "OthelloBoardUI.h"
#include "MainUI.h"

extern GameUIData *mGameUI;

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TOthelloBoardFrame *OthelloBoardFrame;
//---------------------------------------------------------------------------
__fastcall TOthelloBoardFrame::TOthelloBoardFrame(TComponent* Owner)
  : TFrame(Owner)
{
  nSelected = -1;
}
//---------------------------------------------------------------------------
void __fastcall TOthelloBoardFrame::ShapeMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
  TSingleMove nMove;

  nMove.MoveFrom = 0;
  nMove.MoveTo = ((TShape*)Sender)->Tag;
  nMove.bValidMove = true;

  if (MainForm->IsValidMove(nMove))
  {
    ClearSelects();
    MainForm->PlayValidMove(nMove);
  }
}

//---------------------------------------------------------------------------

void __fastcall TOthelloBoardFrame::ShapeMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
  TSingleMove nMove;

  nMove.MoveFrom = 0;
  nMove.MoveTo = ((TShape*)Sender)->Tag;
  nMove.bValidMove = true;

  if (MainForm->IsValidMove(nMove))
  {
    // if there was another move selected before
    if ((nSelected != -1) && (nSelected != (long) nMove.MoveTo))
      ClearSelects();

    ((TShape*)Sender)->Pen->Color = mGameUI->mColourSelectTo;
    nSelected = nMove.MoveTo;
  }
  else
  {
    // no valid move under the stone, so clear the selected move
    ClearSelects();
  }
}

//---------------------------------------------------------------------------
// clear all selected moves on the board

void TOthelloBoardFrame::ClearSelects(void)
{
  int i;

  for (i=0;i<OthelloBoardPanel->ControlCount;i++)
    if (((TShape*)OthelloBoardPanel->Controls[i])->Shape == stRoundSquare)
      if (((TShape*)OthelloBoardPanel->Controls[i])->Pen->Color != mGameUI->mColourSelectNone)
        // this test avoids flickering by only updating when there is something
        // to update
        ((TShape*)OthelloBoardPanel->Controls[i])->Pen->Color = mGameUI->mColourSelectNone;

  nSelected = -1;
}

//---------------------------------------------------------------------------

void TOthelloBoardFrame::SetPieceAt(int nPos, int nColour)
{
  int i, nMax;

  nMax = OthelloBoardPanel->ControlCount;

  for (i=0;i<nMax;i++)
  {
    if (nPos == GetObjectValue(i))
    {
      if (nColour==0)
        ((TShape*)(OthelloBoardPanel->Controls[i]))->Brush->Color = mGameUI->mColourEmpty;
      else if (nColour==1)
        ((TShape*)(OthelloBoardPanel->Controls[i]))->Brush->Color = mGameUI->mColourWhite;
      else
        ((TShape*)(OthelloBoardPanel->Controls[i]))->Brush->Color = mGameUI->mColourBlack;
      return;
    }
  }
}

//---------------------------------------------------------------------------

int TOthelloBoardFrame::GetControlCount(void)
{
  return OthelloBoardPanel->ControlCount;
}

//---------------------------------------------------------------------------

int TOthelloBoardFrame::GetObjectValue(int nIndex)
{
  return OthelloBoardPanel->Controls[nIndex]->Tag;
}


//---------------------------------------------------------------------------
// eof



