//---------------------------------------------------------------------------

#ifndef AlgorithmBaseH
#define AlgorithmBaseH

#include "EngGame.h"

#ifdef _DEBUG
#define MAX_RECURSE_DEBUGLOG 12
#endif

//---------------------------------------------------------------------------

class Algorithm
{
  protected:
    #ifdef _DEBUG
    TSingleMove aMoveArray[MAX_RECURSE_DEBUGLOG];
    void LogMoveList(GameEngine *O1);
    #endif
  public:
    std::string sName[128];
    Algorithm(void);
    virtual int CalcBestMove(GameEngine *O1,const int nLevel,const int nToMove,
                 TSingleMove *nBestMoveHere) = 0;
};

//---------------------------------------------------------------------------

class AlgorithmRandom : public Algorithm
{
  public:
    AlgorithmRandom(void);
    int CalcBestMove(GameEngine *O1,const int nLevel,const int nToMove,
                 TSingleMove *nBestMoveHere);
};

//---------------------------------------------------------------------------

class AlgorithmStaticEval : public Algorithm
{
  public:
    AlgorithmStaticEval(void);
    int CalcBestMove(GameEngine *O1,const int nLevel,const int nToMove,
                 TSingleMove *nBestMoveHere);
};

//---------------------------------------------------------------------------

class AlgorithmBruteForce : public Algorithm
{
  public:
    AlgorithmBruteForce(void);
    int CalcBestMove(GameEngine *O1,const int nLevel,const int nToMove,
                 TSingleMove *nBestMoveHere);
};

//---------------------------------------------------------------------------

class AlgorithmBruteFWin : public Algorithm
{
  public:
    AlgorithmBruteFWin(void);
    int CalcBestMove(GameEngine *O1,const int nLevel,const int nToMove,
                 TSingleMove *nBestMoveHere);
};

//---------------------------------------------------------------------------

class AlgorithmAlphaBeta : public Algorithm
{
  private:
    int AlphaBetaLeaf(GameEngine *O1,
            const int nLevel,const int nToMove,int nAlpha,int nBeta);

  public:
    AlgorithmAlphaBeta(void);
    int CalcBestMove(GameEngine *O1,const int nLevel,const int nToMove,
                 TSingleMove *nBestMoveHere);
};

//---------------------------------------------------------------------------

class AlgorithmNegascout : public Algorithm
{
  private:
    int NegascoutRoot(GameEngine *O1,
                      int nLevel,
                      int nToMove,
                      int nLowerBound,
                      int nUpperBound,
                      TSingleMove *nBestMoveHere);
  public:
    AlgorithmNegascout(void);
    int CalcBestMove(GameEngine *O1,const int nLevel,const int nToMove,
                 TSingleMove *nBestMoveHere);
};

//---------------------------------------------------------------------------
#endif
