// Brute force algorithm file

#ifdef _DEBUG
#include <iostream>
#include <fstream>
#endif

#include "AlgorithmBase.h"

//---------------------------------------------------------------------------

AlgorithmBruteForce::AlgorithmBruteForce(void)
{
  sName->assign("BruteForce");
}

// ---------------------------------------------------------------------------
// Get the best move in a certain position
// go down a level if at this level a best move cannot be found
// tries to maximise the result in this recursion level

int AlgorithmBruteForce::CalcBestMove(GameEngine *oEngine,
                const int nLevel,
                const int nToMove,
                TSingleMove *nBestMoveHere)
{
  int nActualMove;
  TSingleMove nBestMoveDown;  // variable actually not used as it only needs
                              // to be reported to the upper level only
  TMoveList aMoves;
  signed int nScore;
  int i;
  TBoard aSavedBoard;
  int nMoves;

  oEngine->nNodes++;

  #ifdef _DEBUG
  std::string sLevelName = "ABForce::Calc ";

  // in case of debugging, we log every entry in the function
  clog << sLevelName << " Entering : level " <<  nLevel
       << " : player "
       << ( nToMove == 1 ? "White" : "Black" )
       << " to play after : ";

  LogMoveList(oEngine);

  nScore = oEngine->EvaluatePos(nToMove);

  //clog << endl << "(" << oEngine->nNodes << ") score = "
  //  << nScore << endl;

  #endif

  oEngine->PushBoard(&aSavedBoard);
  nMoves=oEngine->GenerateMoves(&aMoves,nToMove);

  if (nMoves)
  {
    for (i=0;i<nMoves;i++)
    {
      oEngine->ApplyMove(aMoves.aMoves[i],nToMove,1);

      #ifdef _DEBUG
        aMoveArray[nLevel] = aMoves.aMoves[i];
        clog << sLevelName << " evaluating : level " << nLevel
            << " : Bestscore = " << nBestMoveHere->nScore
            << " : moves : ";
        LogMoveList(oEngine);
      #endif

      // here recurse or exit
      if (oEngine->IsEnded(nToMove,&nScore))
      {
        #ifdef _DEBUG
          oEngine->LogBoard();
        #endif
      }
      else
      {
        // only go deeper if not ended
        if (nLevel>1)
        {
          nBestMoveDown.init();
          nScore = CalcBestMove(oEngine,nLevel-1,3-nToMove,&nBestMoveDown);
          // adjust score for taking the move to the fastest win
          // first - adjusting towards zero
          if (nScore > 0) nScore --; else if (nScore < 0) nScore ++;
        }
        else
          nScore = oEngine->EvaluatePos(nToMove);
      }
      oEngine->PopBoard(aSavedBoard);

      #ifdef _DEBUG
        clog << sLevelName << " level " <<  nLevel << " : score "
              << nScore << " : " ;
        LogMoveList(oEngine);
        aMoveArray[nLevel].bValidMove = false;
      #endif

      if (nScore>=nBestMoveHere->nScore)
      {
        #ifdef _DEBUG
          clog << sLevelName << " level " <<  nLevel
              << " : better or equal score " << nScore
              << " than score " << nBestMoveHere->nScore << endl;
        #endif
        *nBestMoveHere = aMoves.aMoves[i];
        nBestMoveHere->nScore = nScore;
      }
    }
  }

  #ifdef _DEBUG
  clog << sLevelName << " Exiting : level " << nLevel
        << " : result returned score "
        << nBestMoveHere->nScore << endl;
  #endif

  return -(nBestMoveHere->nScore);
}

// ---------------------------------------------------------------------------
// eof
